"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
var GlobalExceptionFilter_1;
const common_1 = require("@nestjs/common");
const json_logger_service_1 = require("json-logger-service");
const uuid = require("uuid/v1");
const IntegrationError_1 = require("./IntegrationError");
let GlobalExceptionFilter = GlobalExceptionFilter_1 = class GlobalExceptionFilter {
    constructor(sendClientInternalServerErrorCause = false, logAllErrors = false, logErrorsWithStatusCode = []) {
        this.sendClientInternalServerErrorCause = sendClientInternalServerErrorCause;
        this.logAllErrors = logAllErrors;
        this.logErrorsWithStatusCode = logErrorsWithStatusCode;
        this.logger = json_logger_service_1.LoggerFactory.createLogger(GlobalExceptionFilter_1.name);
    }
    static extractIntegrationErrorDetails(error) {
        if (!(error instanceof IntegrationError_1.IntegrationError)) {
            return undefined;
        }
        if (!error.causeError) {
            return undefined;
        }
        if (error.causeError instanceof String) {
            return error.causeError;
        }
        if (!error.causeError.message && !error.causeError.response) {
            return undefined;
        }
        const integrationErrorDetails = {
            message: error.causeError.message,
            details: error.causeError.response && error.causeError.response.data,
        };
        return JSON.stringify({ causeError: integrationErrorDetails });
    }
    catch(exception, host) {
        const ctx = host.switchToHttp();
        const response = ctx.getResponse();
        const request = ctx.getRequest();
        const responseStatus = exception.status ? exception.status : common_1.HttpStatus.INTERNAL_SERVER_ERROR;
        let errorId = undefined;
        let integrationErrorDetails = undefined;
        if (responseStatus === common_1.HttpStatus.INTERNAL_SERVER_ERROR) {
            errorId = uuid();
            integrationErrorDetails = GlobalExceptionFilter_1.extractIntegrationErrorDetails(exception);
            this.logger.error({
                errorId: errorId,
                route: request.url,
                integrationErrorDetails,
                stack: exception.stack && JSON.stringify(exception.stack, ['stack'], 4),
            }, exception.message);
        }
        else if (this.logAllErrors || this.logErrorsWithStatusCode.indexOf(responseStatus) !== -1) {
            this.logger.error({
                route: request.url,
                stack: exception.stack && JSON.stringify(exception.stack),
            }, exception.message);
        }
        response
            .status(responseStatus)
            .json({
            errorId: errorId,
            message: this.getClientResponseMessage(responseStatus, exception),
            integrationErrorDetails: responseStatus === common_1.HttpStatus.INTERNAL_SERVER_ERROR && this.sendClientInternalServerErrorCause ? integrationErrorDetails : undefined,
        });
    }
    getClientResponseMessage(responseStatus, exception) {
        if (responseStatus !== common_1.HttpStatus.INTERNAL_SERVER_ERROR
            || (responseStatus === common_1.HttpStatus.INTERNAL_SERVER_ERROR && this.sendClientInternalServerErrorCause)) {
            return exception.message;
        }
        return 'Internal server error.';
    }
};
GlobalExceptionFilter = GlobalExceptionFilter_1 = __decorate([
    common_1.Catch(),
    __metadata("design:paramtypes", [Boolean, Boolean, Array])
], GlobalExceptionFilter);
exports.GlobalExceptionFilter = GlobalExceptionFilter;
//# sourceMappingURL=GlobalExceptionFilter.js.map